/*
 * Copyright (c) 2019-2020 Actinius
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <drivers/gpio.h>

#define RED_LED_PIN			DT_GPIO_PIN(DT_NODELABEL(red_led), gpios)
#define GREEN_LED_PIN		DT_GPIO_PIN(DT_NODELABEL(green_led), gpios)
#define BLUE_LED_PIN		DT_GPIO_PIN(DT_NODELABEL(blue_led), gpios)

#define BUTTON_NODE    		DT_NODELABEL(button0)
#define BUTTON_GPIO_LABEL	DT_GPIO_LABEL(BUTTON_NODE, gpios)
#define BUTTON_GPIO_PIN		DT_GPIO_PIN(BUTTON_NODE, gpios)
#define BUTTON_GPIO_FLAGS	GPIO_INPUT | DT_GPIO_FLAGS(BUTTON_NODE, gpios)

#define LED_ON 0
#define LED_OFF !LED_ON

typedef enum {
	RED,
	GREEN,
	BLUE,
	MAGENTA,
	CYAN,
	YELLOW
} led_color_t;

static const struct device *gpio_dev;
static struct gpio_callback gpio_cb;

static bool button_pressed = false;

void button_pressed_callback(const struct device *gpiob, struct gpio_callback *cb, gpio_port_pins_t pins)
{
	button_pressed = true;
}

bool init_button(void)
{
	int ret = gpio_pin_configure(gpio_dev, BUTTON_GPIO_PIN, BUTTON_GPIO_FLAGS);
	if (ret != 0) {
        printk("Error %d: failed to configure %s pin %d\n",
            	ret, BUTTON_GPIO_LABEL, BUTTON_GPIO_PIN);
        
		return false;
    }

    ret = gpio_pin_interrupt_configure(gpio_dev,
                                       BUTTON_GPIO_PIN,
                                       GPIO_INT_EDGE_TO_ACTIVE);
    if (ret != 0) {
        printk("Error %d: failed to configure interrupt on %s pin %d\n",
               ret, BUTTON_GPIO_LABEL, BUTTON_GPIO_PIN);

        return false;
    }

    gpio_init_callback(&gpio_cb, button_pressed_callback, BIT(BUTTON_GPIO_PIN));
    gpio_add_callback(gpio_dev, &gpio_cb);

	return true;
}

void init_leds(void)
{
	gpio_pin_configure(gpio_dev, RED_LED_PIN, GPIO_OUTPUT_HIGH);
	gpio_pin_configure(gpio_dev, GREEN_LED_PIN, GPIO_OUTPUT_HIGH);
	gpio_pin_configure(gpio_dev, BLUE_LED_PIN, GPIO_OUTPUT_HIGH);
}

void turn_leds_off(void)
{
	gpio_pin_set(gpio_dev, RED_LED_PIN, LED_OFF);
	gpio_pin_set(gpio_dev, GREEN_LED_PIN, LED_OFF);
	gpio_pin_set(gpio_dev, BLUE_LED_PIN, LED_OFF);
}

void turn_leds_on_with_color(led_color_t color)
{
	switch (color) {
	case RED:
		gpio_pin_set(gpio_dev, RED_LED_PIN, LED_ON);
		break;
	case GREEN:
		gpio_pin_set(gpio_dev, GREEN_LED_PIN, LED_ON);
		break;
	case BLUE:
		gpio_pin_set(gpio_dev, BLUE_LED_PIN, LED_ON);
		break;
	case MAGENTA:
		gpio_pin_set(gpio_dev, RED_LED_PIN, LED_ON);
		gpio_pin_set(gpio_dev, BLUE_LED_PIN, LED_ON);
		break;
	case CYAN:
		gpio_pin_set(gpio_dev, GREEN_LED_PIN, LED_ON);
		gpio_pin_set(gpio_dev, BLUE_LED_PIN, LED_ON);
		break;
	case YELLOW:
		gpio_pin_set(gpio_dev, RED_LED_PIN, LED_ON);
		gpio_pin_set(gpio_dev, GREEN_LED_PIN, LED_ON);
		break;
	}
}

int main(void)
{
	gpio_dev = device_get_binding(DT_LABEL(DT_NODELABEL(gpio0)));

	if (!gpio_dev) {
		printk("Error getting GPIO device binding\r\n");

		return false;
	}

	if (!init_button()) {
		return false;
	}

	init_leds();

	led_color_t color = RED;

	while (1) {
		if (button_pressed) {
			turn_leds_off();
			turn_leds_on_with_color(color);

			color++;
			if (color > 5) {
				color = 0;
			}

			button_pressed = false;
		}

		k_sleep(K_MSEC(100));
	}

	return 0;
}
